from typing import List

from flask import request, current_app
from flask_restx import Namespace, Resource
import os
from CTFd.api.v1.helpers.request import validate_args
from CTFd.api.v1.helpers.schemas import sqlalchemy_to_pydantic
from CTFd.api.v1.schemas import APIDetailedSuccessResponse, APIListSuccessResponse
from CTFd.constants import RawEnum
from CTFd.models import Files, db, Submissions
from CTFd.schemas.files import FileSchema
from CTFd.utils import uploads
from CTFd.utils.decorators import admins_only
from CTFd.utils.helpers.models import build_model_filters
from moviepy.editor import *
from PIL import Image, ExifTags
from werkzeug.utils import secure_filename
import subprocess
import shlex
import json
from CTFd.api.v1 import challenges
from CTFd.api.v1 import submissions


files_namespace = Namespace("files", description="Endpoint to retrieve Files")

FileModel = sqlalchemy_to_pydantic(Files)


class FileDetailedSuccessResponse(APIDetailedSuccessResponse):
    data: FileModel


class FileListSuccessResponse(APIListSuccessResponse):
    data: List[FileModel]


files_namespace.schema_model(
    "FileDetailedSuccessResponse", FileDetailedSuccessResponse.apidoc()
)

files_namespace.schema_model(
    "FileListSuccessResponse", FileListSuccessResponse.apidoc()
)

class FakeRequest(object):
    content = {"none":0}
    args = {}
    access_route = ""
    remote_addr = ""
    def get_json(self):
        return self.content
    def get(self, name):
        try:
            return self.content[name]
        except:
            return None
    def setJson(self, dictJ):
        self.content = dictJ

    def setArgs(self, dictA):
        self.args = dictA

    def __getitem__(self, item):
         return self.content[item]


@files_namespace.route("")
class FilesList(Resource):
    @files_namespace.doc(
        description="Endpoint to get file objects in bulk",
        responses={
            200: ("Success", "FileListSuccessResponse"),
            400: (
                "Une erreur s'est produite en traitant des données fournies ou stockées",
                "APISimpleErrorResponse",
            ),
        },
    )
    @validate_args(
        {
            "type": (str, None),
            "location": (str, None),
            "q": (str, None),
            "field": (
                RawEnum("FileFields", {"type": "type", "location": "location"}),
                None,
            ),
        },
        location="query",
    )
    
    def get(self, query_args):
        q = query_args.pop("q", None)
        field = str(query_args.pop("field", None))
        filters = build_model_filters(model=Files, query=q, field=field)

        files = Files.query.filter_by(**query_args).filter(*filters).all()
        schema = FileSchema(many=True)
        response = schema.dump(files)

        if response.errors:
            return {"success": False, "errors": response.errors}, 400

        return {"success": True, "data": response.data}


    @files_namespace.doc(
        description="Endpoint to get file objects in bulk",
        responses={
            200: ("Success", "FileDetailedSuccessResponse"),
            400: (
                "Une erreur s'est produite en traitant des données fournies ou stockées",
                "APISimpleErrorResponse",
            ),
        },
    )


    def post(self):
        print("/Files Post Request")
        submission_id = -1
        heavyData = False
        maxContentLength = 201000000
        # challenge_id
        # page_id
        #max 200000000 + 1 Mb pour le reste des info de la request
        if int(request.headers.get('Content-Length')) > maxContentLength and int(request.headers.get('Content-Length')) < maxContentLength:
            heavyData = True
        elif int(request.headers.get('Content-Length')) > maxContentLength:
            return {
                "success": False,
                "errors": {
                    "location": ["Le fichier ne peut pas être supérieur à 200 Mo, même avec compression.Veuillez utiliser des outils externes et le partager avec un lien"]
                },
            }, 400

    
        files = request.files.getlist("file")
        
        # Handle situation where users attempt to upload multiple files with a single location
        if len(files) > 1 and request.form.get("location"):
            return {
                "success": False,
                "errors": {
                    "location": ["L'emplacement ne peut pas être spécifié avec plusieurs fichiers"]
                },
            }, 400
        print("Tout devrait bien aller")

        objs = []
        #on genere un fichier de plus qui seras remplacer par la thumbsnail
        files += files[0],
        
        print("files[0]")
        print(files[0])
        vraiType = []
        first = True
        for f in files:
            vraiType += str(f).split('.')[1].split("\'")[0],
            try:
                dico = request.form.to_dict()
                try:
                    del dico["type"]
                    del dico["id"]
                except:
                    pass
                
                dico["first"] = first
                obj = uploads.upload_file(file=f, **dico)
                first = False
                
            except ValueError as e:
                print("return valueError")
                return {
                    "success": False,
                    "errors": {"location": [str(e)]},
                }, 400
            objs.append(obj)
        
        schema = FileSchema(many=True)
        response = schema.dump(objs)
        print("du message")
        print(objs)
        print(schema)
        print(response)
        print(response.data)

        file_info_list = []
        for file in files:
            file_info = {
                "filename": file.filename,
                "content_type": file.content_type
            }
            file_info_list.append(file_info)

        # Step 2: Serialize the information to JSON
        files_json_string = json.dumps(file_info_list)
        
        #we are creating the submissions to make sure nobody can take it if sending a smaller file
        item = FakeRequest()
        if not request.args.get("admin", False):
            print("fausse demande")
            item.setJson({"challenge_id": request.form.get("id"), "submission":"Média en traitement", "type":"None", "response":response.data, "heavyData":heavyData, "vraiType":vraiType, "files_data":files_json_string,})
            print(item.get_json())
            item.access_route = request.access_route
            item.remote_addr = request.remote_addr
            submission_id = challenges.outgoingPost(item)["data"]["submission_id"]
            
        db.session.commit()

        return {"success": True, "data": "Média en traitement"}

@files_namespace.route("/<file_id>")
class FilesDetail(Resource):

    @files_namespace.doc(
        description="Endpoint to get a specific file object",
        responses={
            200: ("Success", "FileDetailedSuccessResponse"),
            400: (
                "Une erreur s'est produite en traitant des données fournies ou stockées",
                "APISimpleErrorResponse",
            ),
        },
    )
    def get(self, file_id):
        
        f = Files.query.filter_by(id=file_id).first_or_404()
        schema = FileSchema()
        response = schema.dump(f)

        if response.errors:
            return {"success": False, "errors": response.errors}, 400

        return {"success": True, "data": response.data}

    @files_namespace.doc(
        description="Endpoint to delete a file object",
        responses={200: ("Success", "APISimpleSuccessResponse")},
    )
    def delete(self, file_id):
        f = Files.query.filter_by(id=file_id).first_or_404()

        uploads.delete_file(file_id=f.id)
        db.session.delete(f)
        db.session.commit()
        db.session.close()

        return {"success": True}
